/* EPITA - IP - S3/4 */
/* question15 */

#define QNUM 15
#define QOPT "graine lines cols"
#define QNOPT 3

# include <assert.h>
# include <stdio.h>
# include <stdlib.h>
# include <unistd.h>

# include "base_test.h"
# include "question15.h"

static
void print_matrix(struct matrix *m) {
  for (size_t i = 0; i < m->lines; i++) {
    printf("|");
    for (size_t j = 0; j < m->cols; j++) {
      printf(" %3d |", m->data[i * m->cols + j]);
    }
    printf("\n");
  }
}

static
struct matrix* build_matrix(size_t l, size_t c) {
  struct matrix *m = malloc(sizeof (struct matrix));
  m->lines = l;
  m->cols = c;
  m->data = calloc(l*c, sizeof (int));
  for (size_t i = 0; i < l; i++) {
    for (size_t j = 0; j < c; j++) {
      m->data[i * c + j] = random() % 100;
    }
  }
  return m;
}

static
void delete_matrix(struct matrix *m) {
  free(m->data);
  free(m);
}

static
struct matrix* build_id_matrix(size_t l) {
  struct matrix *m = malloc(sizeof (struct matrix));
  m->lines = l;
  m->cols = l;
  m->data = calloc(l*l, sizeof (int));

  for (size_t i = 0; i < l; i++)
    m->data[i * l + i] = 1;

  return m;
}

static
void id_test(size_t lines) {
  printf("Test with id matrix:\n");
  struct matrix *A = build_matrix(lines, lines);
  struct matrix *id = build_id_matrix(lines);

  printf("A =\n");
  print_matrix(A);
  printf("id =\n");
  print_matrix(id);

  struct matrix *C;
  printf("C = A * id\n");
  C = matrix_mul(A, id);
  print_matrix(C);

  delete_matrix(A);
  delete_matrix(id);
  delete_matrix(C);

}

static
void random_test(size_t lines, size_t cols) {
  printf("Random tests:\n");
  struct matrix *A = build_matrix(lines, cols);
  struct matrix *B = build_matrix(cols, lines);
  printf("A =\n");
  print_matrix(A);
  printf("B =\n");
  print_matrix(B);

  struct matrix *C;
  printf("C = A * B\n");
  C = matrix_mul(A, B);
  print_matrix(C);

  delete_matrix(A);
  delete_matrix(B);
  delete_matrix(C);
}


int main(int argc,char *argv[])
{
  help_asked(argc,argv,QNUM,QOPT,QNOPT);

  if (argc <= QNOPT)
    {
      fprintf(stderr, "Parametres insuffisants.\n");
      usage(NULL,QNUM,QOPT);
    }
  else
    {
      size_t lines, cols;
      seedInit(argv);
      lines = strtoul(argv[2], NULL, 10);
      cols  = strtoul(argv[3], NULL, 10);
      id_test(lines);
      random_test(lines, cols);
    }
  exit(0);
}
